'Fix fr Album Art Problem im Vista Media Center (VMC)
'
' - Erstellen eines Covers im VMC-Cache fr jeden MP3-Song aus dem Folder.jpg/Ersatz-Cover des Albums (wenn vorhanden)
'
'published by tommce, inspired by Spyn Doctor

' Version   Datum       Bemerkung
' 1.3.2     27.10.2007  Bildschirmausgaben gendert
' 1.3.1     12.10.2007  Erweiterte Fehlerbehandlung
' 1.3       08.10.2007  Optimierung der Analyse von WMP-Bibliothek
'                       - Nur noch Audio-Dateien werden ermittelt
'                       - Smtliche Audio-Typen werden jetzt bearbeitet
'                       Vereinfachung der Steuer-Parameter
' 1.2       10.08.2007  Automatische Verlinkung vom Original-VMC-Cache
' 1.1       06.08.2007  Weitere Optimierungen
'                       - WMP Cache wird nicht mehr bearbeitet
'                       - Verlinkung aller Titel-Cover auf eine Datei je Album  
' 1.0       16.07.2007  Erste Version
'
'
'Im VMC-Cache wird fr jede Audio-Datei ein Link (Hardlink oder Symbolischer Link) auf das Album-Cover im Albumordner erzeugt.
'Bei der Verwendung von Hardlinks muss der Cache-Ordner auf der selben Partition liegen wie die Album-Cover.
'Dies gelingt natrlich nicht, wenn die Mediendateien auf mehreren Partitionen verteilt liegen.
'Dafr knnen diese Links auch problemlos von Nicht-Administratoren eingerichtet werden.
'Symbolische Links knnen auf beliebige Ordner/Dateien verweisen.
'Berechtigt zum Erstellen sind zunchst nur Administratoren. Die Berechtigung lsst sich ber eine Gruppenrichtlinie erweitern.
'Der Editor hierfr wird mit gpedit.msc gestartet.
'- Computerkonfiguration\Administrative Vorlagen\System\NTFS-Dateisystem\Auswertung symbolischer Links selektiv zulassen
'
'Der Standardname fr das Cover im Albumordner lautet "Folder.jpg". Dieser Name wird auch vom Windows Media Player (WMP) verwendet.
'Soll mit einem alternativen Namen gearbeitet werden, so ist die Variable "strAlbumCover" anzupassen.
'Wenn der Standardname nicht verwendet wird, wird die Datei Folder.jpg aus der Ersatzdatei durch Kopieren erzeugt und umgekehrt!
'Dadurch ist gewhrleistet, dass der WMP ein Cover vorfindet.
'
'Die Art der Cover-Behandlung (Hardlink, Symbolischer Link, Kopie) wird ber die Variable "intMode" gesteuert.
'
'Der Original-VMC-Cache sollte als Link (Junction) eingerichtet werden. Dies wird automatisch durch das Skript erledigt.
'Der Befehl hierfr sieht sinngem folgendermaen aus:
'mklink /j <Pfad zum VMC-Cache> "C:\Users\<Benutzername>\AppData\Local\Microsoft\ehome\Art Cache"
'<Pfad zum VMC-Cache> und <Benutzername> sind passend zu ersetzen.
'
'
'Ausfhrliche Zusatz-Erluterungen sind hier zu finden:
'http://www.mce-community.de/forum/index.php?showtopic=16655&view=findpost&p=119880
'http://www.mce-community.de/forum/index.php?showtopic=16655&view=findpost&p=140926
'http://www.mce-community.de/forum/index.php?showtopic=16655&view=findpost&p=142956
'
'
Option Explicit

'Parameter-Variablen
Dim strAlbumCover, strVMCCache, intMode

'Modi fr Cover-Behandlung
Const MODE_HARDLINK = 0     'Hardlink erzeugen
Const MODE_SYMLINK = 1      'Symbolischen Link erzeugen
Const MODE_COPY = 2         'Kopieren

'*******************************************************************************
'*******************************************************************************
'****************** Konfiguriere ab hier / Configure from here *****************
'strAlbumCover (Zeichenkette/String): Cover-Name oder "Folder.jpg"
'Bsp.:
'strAlbumCover = "Cover.jpg"
strAlbumCover = "Cover.jpg"

'strVMCCache (Zeichenkette/String): Vollstndiger Pfad zum VMC-Cache
'Bsp.:
'strVMCCache = "D:\Art Cache"
strVMCCache = "D:\Art Cache"

'intMode (Integer-Zahl/Number): Cover-Behandlung
'  MODE_HARDLINK : Hardlink erzeugen
'  MODE_SYMLINK  : Symbolischen Link erzeugen
'  MODE_COPY     : Kopieren
'Bsp.:
'intMode = MODE_HARDLINK
intMode = MODE_HARDLINK
'***************** Konfiguriere bis hier / Configure until here ****************
'*******************************************************************************
'*******************************************************************************

On Error Resume Next

'Weitere Variablen
Dim FSO, WshShell
Dim MyWMP, MyPlaylist
Dim intIndex, intVMCCovers, intWarnings
Dim strFolder, strFolderJPG, strSrcCover, strVMCCover, strCommand
Dim datStart, datEnd, intDuration

Set FSO = Wscript.CreateObject("Scripting.FileSystemObject")
Set WshShell = Wscript.CreateObject("Wscript.Shell")

'Initialisierung
intVMCCovers = 0
intWarnings = 0
datStart = Now()

Wscript.Echo "Album Art Fix 1.3.2 (fr Vista Media Center)"
Wscript.Echo ""
Wscript.Echo "Album-Cover : " & strAlbumCover
Wscript.Echo "VMC-Cache   : " & strVMCCache
Select Case intMode
Case MODE_COPY
    Wscript.Echo "Covers werden kopiert, nicht verlinkt!"
Case MODE_SYMLINK
    Wscript.Echo "Verwende Symbolische Links!"
Case Else   'MODE_HARDLINK
    intMode = MODE_HARDLINK
    Wscript.Echo "Verwende Hardlinks!"
End Select

If Not CheckCacheFolder(strVMCCache) Then
    Wscript.Echo "Ausfhrung beendet wegen eines Problems mit dem VMC-Cache!"
    Wscript.Quit    
End If

Wscript.Echo ""
Wscript.Echo "Scanne WMP-Bibliothek..."

'Audio-Dateien aus WMP-Bibliothek ermitteln
Set MyWMP = CreateObject("WMPlayer.OCX")
Set MyPlaylist = MyWMP.mediaCollection.getByAttribute("MediaType", "audio")
If Err <> 0 Then
    ShowError
    Wscript.Echo "Ausfhrung wird beendet wegen eines Problems mit der WMP-Bibliothek!"
    Wscript.Quit
End If

For intIndex = 0 to MyPlaylist.Count - 1  
    With MyPlaylist.Item(intIndex)
        If .getItemInfo("TrackingID") <> "" Then
            If FSO.FileExists(.sourceURL) Then
                strFolder = FSO.GetFile(.sourceURL).ParentFolder
                strFolderJPG = strFolder & "\Folder.jpg"
                strSrcCover = strFolder & "\" & strAlbumCover
                strVMCCover = strVMCCache & "\AlbumArt_" & .getItemInfo("TrackingID") & "_Large.jpg"
    
                If UCase(strAlbumCover) <> "FOLDER.JPG" Then
                'Alternativer Cover-Name
                    If FSO.FileExists(strSrcCover) And Not FSO.FileExists(strFolderJPG) Then
                    'Folder.jpg erzeugen
                        Wscript.Echo "Erstelle Folder.jpg in """ & strFolder & """"
                        FSO.CopyFile strSrcCover, strFolderJPG
                    ElseIf FSO.FileExists(strFolderJPG) And Not FSO.FileExists(strSrcCover) Then
                    'Alternatives Cover erzeugen
                        Wscript.Echo "Erstelle " & strAlbumCover & " in """ & strFolder & """"
                        FSO.CopyFile strFolderJPG, strSrcCover
                    End If

                    If Err <> 0 Then
                        intWarnings = intWarnings + 1
                        ShowError
                        Err.Clear
                    End If
                End If
      
                If FSO.FileExists(strSrcCover) And Not FSO.FileExists(strVMCCover) Then
                'Fehlendes Cover in VMC-Cache einfgen
                    Wscript.Echo "Fge Cover zum Cache fr """ & .name & """ in """ & strFolder & """"
                    intVMCCovers = intVMCCovers + 1
                    Select Case intMode
                    Case MODE_COPY
                    'Cover kopieren
                        FSO.CopyFile strSrcCover, strVMCCover
                    Case MODE_SYMLINK
                    'Symbolischen Link erzeugen
                        strCommand = "cmd /c mklink """ & strVMCCover & """ """ & strSrcCover & """" 
                        WshShell.Run strCommand, 0, True
                    Case MODE_HARDLINK
                    'Hardlink erzeugen
                        strCommand = "cmd /c mklink /h """ & strVMCCover & """ """ & strSrcCover & """" 
                        WshShell.Run strCommand, 0, True
                    Case Else
                        Wscript.Echo "Unbekannter Modus: " & intMode & "!"
                    End Select

                    If Err <> 0 Then
                        intWarnings = intWarnings + 1
                        ShowError
                        Err.Clear
                    End If
                End If                
            Else
                intWarnings = intWarnings + 1
                Wscript.Echo "Datei nicht gefunden: """ & .sourceURL & """"
            End If
        Else
            intWarnings = intWarnings + 1
            Wscript.Echo "TrackingID fehlt fr """ & .sourceURL & """"
        End If
    End With

    If ((intIndex + 1) Mod 1000) = 0 Then
        Wscript.Echo (intIndex + 1) & " Eintrge wurden bis jetzt untersucht! Bitte noch etwas Geduld..."
    End If
Next

Wscript.Echo ""
If intWarnings > 0 Then Wscript.Echo intWarnings & " Warnungen aufgetreten!"
Wscript.Echo intVMCCovers & " Cover(s) zum Cache hinzugefgt!"

datEnd = Now()
intDuration = DateDiff("s", datStart, datEnd)
Wscript.Echo "Start : " & datStart
Wscript.Echo "Ende  : " & datEnd
Wscript.Echo "Dauer : " & (intDuration \ 60) & " min. " & (intDuration - (intDuration \ 60) * 60) & " sek."
Wscript.Echo "Fertig!"
Wscript.Quit


Function CheckCacheFolder(ByVal strVMCCache)
'berprft (und erstellt) den VMC-Cache
Dim intRet
Dim blnOk
Dim strVMCCacheOrig, strTempName
Dim MyTempFile

    On Error Resume Next

    blnOk = False
    
    strVMCCacheOrig = Wshshell.ExpandEnvironmentStrings("%USERPROFILE%") & "\AppData\Local\Microsoft\ehome\Art Cache"

    If Not FSO.FolderExists(strVMCCache) Then
    'Cache-Ordner existiert noch nicht
        intRet = MsgBox("Soll der originale VMC-Cache auf den NEUEN Ordner """ & strVMCCache & """ verlinkt werden?", vbOKCancel + vbQuestion)
        If intRet = vbOK Then
        'Ordner und Link erstellen
            WScript.Echo "Erstelle Ordner """ & strVMCCache & """"
            FSO.CreateFolder strVMCCache
            If Err = 0 Then
                blnOk = LinkCacheFolder(strVMCCache, strVMCCacheOrig)
            Else
                ShowError
            End If
        End If            
    Else
    'Cache-Ordner existiert, Link berprfen
        strTempName = FSO.GetTempname
        Set MyTempFile = FSO.CreateTextFile(strVMCCacheOrig & "\" & strTempName, True)
        MyTempFile.Close
        If Err = 0 Then
            If FSO.FileExists(strVMCCache & "\" & strTempName) Then
            'VMC-Cache bereits verlinkt
                FSO.DeleteFile strVMCCacheOrig & "\" & strTempName
                blnOk = True
            Else
                FSO.DeleteFile strVMCCacheOrig & "\" & strTempName
                intRet = MsgBox("Soll der originale VMC-Cache auf den vorhandenen Ordner """ & strVMCCache & """ verlinkt werden?", vbOKCancel + vbQuestion)
                If intRet = vbOK Then
                    blnOk = LinkCacheFolder(strVMCCache, strVMCCacheOrig)
                End If            
            End If
        Else
            ShowError
        End If
    End If
    
    CheckCacheFolder = blnOk
    Err.Clear
    
End Function


Function LinkCacheFolder(ByVal strVMCCache, ByVal strVMCCacheOrig)
'Verlinkt den originalen VMC-Cache auf einen anderen Ordner
Dim blnOk
Dim strCommand, strBackupFolder

    On Error Resume Next
    
    blnOk = False
    
    'Originalen VMC-Cache umbenennen (Backup)
    strBackupFolder = strVMCCacheOrig & ".backup"
    If FSO.FolderExists(strBackupFolder) Then FSO.DeleteFolder(strBackupFolder)
    FSO.MoveFolder strVMCCacheOrig, strBackupFolder
    
    If Err = 0 Then
        'Verlinkung fr den VMC-Cache erstellen
        Wscript.Echo "Erstelle Link zu """ & strVMCCache & """ fr """ & strVMCCacheOrig & """"
        strCommand = "cmd /c mklink /j """ & strVMCCacheOrig & """ """ & strVMCCache & """" 
        WshShell.Run strCommand, 0, True
        If Err = 0 Then
            blnOk = True
        Else
            ShowError
        End If
    Else
        ShowError
    End If

    LinkCacheFolder = blnOk
    Err.Clear
    
End Function


Sub ShowError()
    Wscript.Echo "*** Fehlerbechreibung Beginn ***"
    WScript.Echo "Fehler-Nr. " & Err
    Wscript.Echo Err.Description 
    Wscript.Echo "*** Fehlerbechreibung Ende   ***"
End Sub
